package com.gci.cdenv.data.consume.app.modules.config;

import io.lettuce.core.ClientOptions;
import io.lettuce.core.SocketOptions;
import io.lettuce.core.resource.ClientResources;
import io.lettuce.core.resource.DefaultClientResources;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisPassword;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

import java.time.Duration;

/**
 * @author lyz
 * @date 2023/11/23 15:03
 */
@Configuration
public class RedisConfig {

    @Value("${redis.first.host}")
    private String firstRedisHost;

    @Value("${redis.first.port}")
    private int firstRedisPort;

    @Value("${redis.first.password}")
    private String firstRedisPassword;

    @Value("${redis.second.host}")
    private String secondRedisHost;

    @Value("${redis.second.port}")
    private int secondRedisPort;

    @Value("${redis.second.password}")
    private String secondRedisPassword;

    @Value("${redis.three.host}")
    private String threeRedisHost;

    @Value("${redis.three.port}")
    private int threeRedisPort;

    @Value("${redis.three.password}")
    private String threeRedisPassword;


    @Bean(name = "firstRedisConnectionFactory")
    public RedisConnectionFactory firstRedisConnectionFactory() {
        return createLettuceConnectionFactory(firstRedisHost, firstRedisPort, firstRedisPassword);
    }

    @Bean(name = "redisTemplate")
    public StringRedisTemplate firstRedisTemplate(@Qualifier("firstRedisConnectionFactory") RedisConnectionFactory redisConnectionFactory) {
        return createRedisTemplate(redisConnectionFactory);
    }

    @Bean(name = "secondRedisConnectionFactory")
    public RedisConnectionFactory secondRedisConnectionFactory() {
        return createLettuceConnectionFactory(secondRedisHost, secondRedisPort, secondRedisPassword);
    }

    @Bean(name = "redisTemplate2")
    public StringRedisTemplate  secondRedisTemplate(@Qualifier("secondRedisConnectionFactory") RedisConnectionFactory redisConnectionFactory) {
        return createRedisTemplate(redisConnectionFactory);
    }

    @Bean(name = "threeRedisConnectionFactory")
    public RedisConnectionFactory threeRedisConnectionFactory() {
        return createLettuceConnectionFactory(threeRedisHost, threeRedisPort, threeRedisPassword);
    }

    @Bean(name = "redisTemplate3")
    public StringRedisTemplate  threeRedisTemplate(@Qualifier("threeRedisConnectionFactory") RedisConnectionFactory redisConnectionFactory) {
        return createRedisTemplate(redisConnectionFactory);
    }

    private LettuceConnectionFactory createLettuceConnectionFactory(String host, int port, String password) {
        ClientResources clientResources = DefaultClientResources.builder().build();

        LettuceClientConfiguration lettuceClientConfiguration = LettuceClientConfiguration.builder()
                .clientResources(clientResources)
                .commandTimeout(Duration.ofSeconds(10))
                .shutdownTimeout(Duration.ofMillis(100))
                .build();
        RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration(host, port);
        redisStandaloneConfiguration.setPassword(RedisPassword.of(password));

        return new LettuceConnectionFactory(
                redisStandaloneConfiguration,
                lettuceClientConfiguration
        );
    }

    private StringRedisTemplate  createRedisTemplate(RedisConnectionFactory redisConnectionFactory) {
        StringRedisTemplate redisTemplate = new StringRedisTemplate();
        redisTemplate.setConnectionFactory(redisConnectionFactory);
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        return redisTemplate;
    }
}
