package com.gci.cdenv.data.consume.app.modules.gps.dto;

import com.gci.cdenv.data.consume.app.modules.common.fence.Coordinate;
import lombok.Data;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.*;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author lyz
 * @date 2023/9/6 14:01
 */
@Data
public class FenceDTO {
    /**
     * 围栏id
     */
    private String fenceId;
    /**
     * 围栏名称
     */
    private String fenceName;

    /**
     * 围栏类型 1-停场 2-行驶
     */
    private Integer fenceType;

    /**
     * 开始时间
     */
    private String startTime;

    /**
     * 结束时间
     */
    private String endTime;

    /**
     * 是否跨天
     */
    private Integer isOvernight;

    /**
     * 车辆驶出围栏多少分钟后开始告警，整数
     */
    private Integer alertDelayMinutes;
    /**
     * 围栏坐标list
     */
    private List<Coordinate> coordinateList;

    /**
     * 获取围栏告警的预计结束时间
     * @param gpsTime
     * @return
     */
    public long estimateTime(Date gpsTime){
        try {
            LocalDate localDate = gpsTime.toInstant().atZone(ZoneId.systemDefault()).toLocalDate();
            LocalTime end = LocalTime.parse(endTime);
            LocalDateTime estimatedEndTime = LocalDateTime.of(localDate, end);
            if (Objects.equals(isOvernight,1)) {
                // 如果跨夜，将结束时间加一天
                estimatedEndTime = estimatedEndTime.plusDays(1);
            }

            Instant instant = estimatedEndTime.toInstant(ZoneOffset.ofHours(8));
            return instant.toEpochMilli();
        } catch (Exception e) {
            e.printStackTrace();
            // 解析时间出错，返回false
            return System.currentTimeMillis();
        }
    }

    /**
     * 判断当前gps时间是否在围栏限制的生效时间范围内
     * @param gpsTime
     * @return
     */
    public boolean isWithinFenceTime(Date gpsTime){
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("HH:mm");
            String formatGpsTime = sdf.format(gpsTime);
            LocalTime current = LocalTime.parse(formatGpsTime);
            LocalTime start = LocalTime.parse(startTime);
            LocalTime end = LocalTime.parse(endTime);

            if (Objects.nonNull(isOvernight) && 1 == isOvernight ) {
                // 如果跨夜，将结束时间加一天
                end = end.plus(Duration.ofHours(24));
            }
            if (start.isBefore(end)) {
                // 同一天内的时间段
                return !current.isBefore(start) && !current.isAfter(end);
            } else {
                // 跨天的时间段，例如 22:00 到 06:00
                return !current.isBefore(start) || !current.isAfter(end);
            }
        } catch (Exception e) {
            e.printStackTrace();
            // 解析时间出错，返回false
            return false;
        }
    }
}
