package com.gci.cdenv.data.consume.app.modules.gps.dto;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.format.annotation.DateTimeFormat;

import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;

/**
 * @author lyz
 * @date 2023/9/1 9:05
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class GpsData {
    /**
     * 车牌号
     */
    private String vpno;

    /**
     * 车牌颜色，可选
     */
    private String vpcolor;

    /**
     * 定位终端识别号
     */
    private String vtkey;

    /**
     * 定位时间
     */
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date gpstime;

    /**
     * 经度
     */
    private BigDecimal lon;

    /**
     * 纬度
     */
    private BigDecimal lat;

    /**
     * GPS速度
     */
    private BigDecimal speed;

    /**
     * 方向
     */
    private int dir;

    /**
     * 高程
     */
    private BigDecimal altitude;

    /**
     * 车辆状态
     */
    private Long vstate;

    /**
     * 车辆告
     */
    private Long valarm;

    /**
     * 定位有效状态
     */
    private int iseff;

    /**
     * ACC开关状态
     */
    private int acc;

    /**
     * 近光灯 0-关闭 1-开启 -1无效
     */
    private Integer lowBeam;

    /**
     * 远光灯 0-关闭 1-开启  -1无效
     */
    private Integer highBeam;

    /**
     * 是否倒挡 0-否 1-是
     */
    private Integer reverseGear;

    public int getWorkStatus(Integer flm) {
        //近光或者远光有一个开启则为作业状态
        if (1 == lowBeam || 1 == highBeam) {
            return 1;
        }
        //如果为福龙马，则倒挡状态为1 也是工作状态
        if (1 == flm && 1 == reverseGear) {
            return 1;
        }
        if (1 == flm) {
            //如果都为0，则为未作业，否则数据无效
            if (0 == highBeam && 0 == lowBeam && 0 == reverseGear) {
                return 0;
            }
        } else {
            if (0 == highBeam && 0 == lowBeam) {
                return 0;
            }
        }
        return -1;
    }

    public boolean isValid() {
        return 1 == iseff;
    }

    public BigDecimal getLat() {
        return Objects.nonNull(lat) ? lat.setScale(6, BigDecimal.ROUND_HALF_UP) : null;
    }

    public BigDecimal getLon() {
        return Objects.nonNull(lon) ? lon.setScale(6, BigDecimal.ROUND_HALF_UP) : null;
    }
}
