package com.gci.cdenv.data.consume.app.modules.gps.dto;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.annotation.JSONField;
import com.gci.cdenv.data.consume.app.modules.common.util.PlateNumberUtil;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.Date;
import java.util.Objects;

@Data
public class VehicleInfoDTO {
    /**
     * 项目Id
     */
    @JSONField(name = "projectId")
    private String projectId;
    /**
     * 项目名称
     */
    @JSONField(name = "projectName")
    private String projectName;
    /**
     * 车牌号
     */
    @JSONField(name = "plateNumber")
    private String plateNumber;
    /**
     * vin码
     */
    @JSONField(name = "vinCode")
    private String vinCode;
    /**
     * 主动安全设备id
     */
    @JSONField(name = "activeSafetyDeviceId")
    private String activeSafetyDeviceId;
    /**
     * 定位终端id
     */
    @JSONField(name = "locationTerminalId")
    private String locationTerminalId;
    /**
     * 总电量
     */
    @JSONField(name = "batteryCapacity")
    private Integer batteryCapacity;

    /**
     * 车辆最后的gps更新时间
     */
    private Date gpsTime;

    /**
     * 车辆支持的gps采集来源 支持三电则为三电，不支持三电
     */
    private Integer supportEic;

    /**
     * 车辆品牌
     */
    private String vehicleBrand;

    /**
     * 车辆种类
     */
    private String vehicleType;

    /**
     * 通过车牌号判断是否是福龙马的车
     * @return
     */
    public  Integer isFuLongMa() {
        if (ObjectUtil.equal("3",vehicleBrand)) {
            return 1;
        } else {
            return 0;
        }
    }

    public boolean needUpdate(Date collectTime) {
        if (Objects.isNull(this.gpsTime)) {
            return true;
        }
        boolean isValid = isValidByTime(collectTime);
        //当为有效记录且 gps时间在采集时间之前，才会返回true
        return isValid && this.gpsTime.before(collectTime);
    }

    /**
     * 根据采集时间判断该条数据是否有效。
     *
     * @param collectTime
     * @return
     */
    public boolean isValidByTime(Date collectTime) {
        if (Objects.isNull(collectTime)) {
            return false;
        }
        //计算采集时间跟当前时间差，剔除采集时间穿越到未来的情况，按理来说采集时候不可能在当前时间之后。
        long timeDifference = collectTime.getTime() - System.currentTimeMillis();
        long minutesDifference = 5 * 60 * 1000;
        //暂定允许5分钟误差，如果时间差大于5分钟，则返回false
        if (timeDifference > minutesDifference) {
            return false;
        }
        return true;
    }

    /**
     * 判断车辆是否支持三电设备并在过去10分钟内有GPS数据更新
     *
     * @return 如果满足条件则返回true，否则返回false
     */
    public boolean isThreeElectricWithRecentGPSUpdate(Integer minute) {
        if (Objects.isNull(gpsTime) || Objects.isNull(supportEic)) {
            return true;
        }
        // 检查车辆是否支持三电设备,不支持直返回需要处理。
        if (supportEic == 0) {
            return true;
        }
        // 然后检查GPS数据是否更新在过去5分钟内
        long timeDifference = System.currentTimeMillis() - gpsTime.getTime();
        long tenMinutesInMillis = minute * 60 * 1000;
        return timeDifference > tenMinutesInMillis;
    }
}
