package com.gci.cdenv.data.consume.app.modules.gps.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Objects;

import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.Data;
import lombok.Getter;
import lombok.Setter;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * <p>
 * 车辆实时信息表
 * </p>
 *
 * @author lyz
 * @since 2023-08-31
 */
@Data
@TableName("tb_gis_runningbus")
public class GisRunningbus implements Serializable {

    private static final long serialVersionUID = 1L;

    @TableId(value = "id", type = IdType.ASSIGN_ID)
    private String id;

    /**
     * 车牌号
     */
    private String plateNumber;

    /**
     * 定位终端标识
     */
    private String gpsNo;

    /**
     * 主动安全设备标识
     */
    private String adasNo;

    /**
     * 车辆状态  1-启动  2-熄火 3-其它  -99-无效 -100：异常
     */
    private Integer status;

    /**
     * vin码
     */
    private String vinCode;

    /**
     * gps采集时间
     */
    @JsonFormat(timezone = "GMT+8",pattern = "yyyy-MM-dd HH:mm:ss")
    @DateTimeFormat(pattern="yyyy-MM-dd HH:mm:ss")
    private Date gpsTime;

    /**
     * 纬度
     */
    private BigDecimal lat;

    /**
     * 经度
     */
    private BigDecimal lon;

    /**
     * 高德纬度
     */
    private BigDecimal gdLat;

    /**
     * 高德经度
     */
    private BigDecimal gdLon;

    /**
     * 速度 单位km/h
     */
    private BigDecimal speed;

    /**
     * 方向角
     */
    private String dir;

    /**
     * 高程
     */
    private Integer altitude;

    /**
     * acc状态 0-熄火 1-点火
     */
    private Integer acc;

    /**
     * 设备是否在线 0-离线 1-在线
     */
    private Integer isOnline;

    /**
     * 故障报警 0-正常 1-异常
     */
    private Integer faultAlarm;

    /**
     * 围栏告警 0-正常 1-异常
     */
    private Integer fenceAlarm;

    /**
     * 超速告警 0-正常 1-异常
     */
    private Integer speedingAlarm;

    /**
     * 主动安全告警-0-正常 1-异常
     */
    private Integer safetyAlarm;

    /**
     * 充电状态  1-停车充电、2-行驶充电、3-未充电  4-充电完成、-99-无效、-100-异常
     */
    private Integer chargeStatus;

    /**
     * 运行模式 1-纯电 2-混动 3-燃油 -99-无效、-100-异常
     */
    private Integer operatingMode;

    /**
     * 累计里程 单位km
     */
    private BigDecimal totalMileage;

    /**
     * 总电压 单位V
     */
    private BigDecimal totalVoltage;

    /**
     * 总电流 单位A
     */
    private BigDecimal totalCurrent;

    /**
     * 剩余电量比例 %
     */
    private Integer stateOfCharge;

    /**
     * dc状态 1-工作 2-断开
     */
    private Integer dcStatus;

    /**
     * 挡位
     */
    private String gearPosition;

    /**
     * 主动安全故障标识
     */
    private String warnNo;

    /**
     * 所属机构
     */
    private String sysOrgCode;

    /**
     * 所属项目名称
     */
    private String projectId;

    /**
     * 整车数据
     */
    private String vehicleJson;

    /**
     * 发动机数据
     */
    private String engineJson;

    /**
     * 报警数据
     */
    private String alarmDataJson;

    /**
     * 驱动机数据
     */
    private String electricMotorJson;

    /**
     * 极值数据
     */
    private String extremeDataJson;

    /**
     * 位置数据
     */
    private String locationJson;

    /**
     * 来源 1-808 2-eic 3-1078
     */
    private Integer recordSource;

    /**
     * 是否支持三电终端，有三电数据上来则为1.默认为0
     */
    private Integer supportEic = 0;

    /**
     * 是否作业 0-否 1-是 -1-无效
     */
    private Integer isWork;


    /**
     * 808作业记录时间
     */
    @TableField(exist = false)
    private Date workRecordTime;

    /**
     *判断是否超时，是则返回ture
     * @return
     */
    public boolean offOnlineTimeOut(Integer minute){
        long  currentTime = System.currentTimeMillis();
        long timeDifference = currentTime - gpsTime.getTime();
        long tenMinutesInMillis = minute * 60 * 1000;
        return timeDifference>tenMinutesInMillis;
    }

    /**
     * 如果gps时间 大于最后记录的作业状态时间为3分钟以上，则需要更新缓存信息里的作业状态
     * @param gpsTime
     * @return
     */
    public boolean needUpdateWorkStatus(Date gpsTime){
        if(Objects.isNull(workRecordTime)){
            return false;
        }
        long diff = gpsTime.getTime() - workRecordTime.getTime();
        long tenMinutesInMillis = 5 * 60 * 1000;
        return diff>tenMinutesInMillis;
    }
}
