package com.gci.cdenv.data.consume.app.modules.gps.service;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.gci.cdenv.data.consume.app.modules.common.util.CoordinateConverterUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.LatLonPoint;
import com.gci.cdenv.data.consume.app.modules.gps.constant.EventType;
import com.gci.cdenv.data.consume.app.modules.gps.dto.*;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisActiveSafetyAlarm;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisRunningbus;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.CommonMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisActiveSafetyAlarmMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisRunningbusMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * 主动安全
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class GisActiveSafetyService {

    @Autowired
    private CommonMapper commonMapper;
    @Autowired
    private GisActiveSafetyAlarmMapper safetyAlarmMapper;
    @Autowired
    private GisRunningbusMapper gisRunningbusMapper;

    @Autowired
    private VehicleBindDriverService vehicleBindDriverService;

    public void gisActiveSafety(ActiveSafetyDto dto) {
        //主动安全设备标识id
        String safetyId = dto.getDeviceId();
        //1、车牌、报警事件标识、一级类型、事件类型 多个条件确定一条记录
        //2、对应数据若已保存一条仅有开始时间的则根据状态判断是否结束
        //3、若没有仅开始数据，则根据状态为开始新增一条数据
        if (Objects.isNull(dto.getDeviceId()) || Objects.equals("", dto.getDeviceId())) {
            log.error("主动安全告警消费数据，deviceId为空，标识号：{}", dto.getWarnNo());
            return;
        }
        VehicleInfoDTO vehicleInfoDTO = commonMapper.getVehicleInfoBySafetyId(safetyId);
        if (Objects.nonNull(vehicleInfoDTO)) {
            LambdaQueryWrapper<GisActiveSafetyAlarm> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(GisActiveSafetyAlarm::getPlateNumber, vehicleInfoDTO.getPlateNumber())
                    .eq(GisActiveSafetyAlarm::getWarnNo, dto.getWarnNo());
//                    .eq(GisActiveSafetyAlarm::getPrimaryType,dto.getPrimaryType())
//                    .eq(GisActiveSafetyAlarm::getEventType,dto.getEventType())
//                    .isNull(GisActiveSafetyAlarm::getAlertEndTime);
            GisActiveSafetyAlarm safetyAlarm = safetyAlarmMapper.selectOne(queryWrapper);
            if (Objects.isNull(safetyAlarm)) {
                //判断标志状态  flagStatus-0-可用 1-开始 2-结束
                if (dto.getFlagStatus() == 1 || dto.getFlagStatus() == 0) {
                    GisActiveSafetyAlarm newAlarm = new GisActiveSafetyAlarm();
                    newAlarm.setAdasNo(dto.getDeviceId());
                    newAlarm.setWarnNo(dto.getWarnNo());
                    newAlarm.setFlagStatus(dto.getFlagStatus());
                    newAlarm.setPrimaryType(dto.getPrimaryType());
                    newAlarm.setEventType(dto.getEventType().toString());
                    newAlarm.setEventName(EventType.getName(dto.getEventType()));
                    newAlarm.setPrecedingVehicleSpeed(dto.getPrecedingVehicleSpeed());
                    newAlarm.setPrecedingDistance(dto.getPrecedingDistance());
                    newAlarm.setDeviationType(dto.getDeviationType());
                    newAlarm.setRoadsignType(dto.getRoadSignType());
                    newAlarm.setFatigueLevel(dto.getFatigueLevel());
                    newAlarm.setAlertLevel(dto.getAlertLevel());
                    newAlarm.setVehicleSpeed(BigDecimal.valueOf(dto.getVehicleSpeed()));
                    newAlarm.setLat(dto.getLat());
                    newAlarm.setLon(dto.getLon());

                    DriverSignInDto driverDto =  vehicleBindDriverService.getCacheDriverInfo(vehicleInfoDTO.getPlateNumber());
                    if(Objects.nonNull(driverDto)){
                        newAlarm.setDriverId(driverDto.getDriverId());
                        newAlarm.setDriverName(driverDto.getDriverName());
                    }
                    //转换高德信息
                    LatLonPoint point = CoordinateConverterUtil.toWGSPoint(dto.getLat().doubleValue(), dto.getLon().doubleValue());
                    newAlarm.setGdLon(new BigDecimal(point.getLongitude()));
                    newAlarm.setGdLat(new BigDecimal(point.getLatitude()));
                    newAlarm.setAlertStartTime(dto.getAlertTime());
                    newAlarm.setAccStatus(dto.getAccStatus());
                    newAlarm.setLeftTurnStatus(dto.getLeftTurnStatus());
                    newAlarm.setRightTurnStatus(dto.getRightTurnStatus());
                    newAlarm.setWiperStatus(dto.getWiperStatus());
                    newAlarm.setBrakeStatus(dto.getBrakeStatus());
                    newAlarm.setCardInsertStatus(dto.getCardInsertStatus());

                    newAlarm.setPlateNumber(vehicleInfoDTO.getPlateNumber());
                    newAlarm.setProjectId(vehicleInfoDTO.getProjectId());
                    newAlarm.setProjectName(vehicleInfoDTO.getProjectName());
                    newAlarm.setAlertEndTime(dto.getAlertTime());
                    newAlarm.setCreateTime(new Date());
                    newAlarm.setUpdateTime(new Date());
                    newAlarm.setDriverId("");
                    newAlarm.setDriverName("");
                    safetyAlarmMapper.insert(newAlarm);
                }
            } else {
                if (dto.getFlagStatus() == 2) {
                    //开始时间要不超过结束时间
                    if (DateUtil.compare(safetyAlarm.getAlertStartTime(), dto.getAlertTime()) <= 0) {
                        safetyAlarm.setFlagStatus(dto.getFlagStatus());
                        safetyAlarm.setUpdateTime(new Date());
                        safetyAlarm.setAlertEndTime(dto.getAlertTime());
                        safetyAlarmMapper.updateById(safetyAlarm);
                    }
                }
            }
            //更新主动安全告警字段回runningBus  主动安全告警-0-正常 1-异常
            GisRunningbus runningbus = gisRunningbusMapper.selectByPlateNumber(vehicleInfoDTO.getPlateNumber());
            if (Objects.nonNull(runningbus)) {
                if (dto.getFlagStatus() == 1) {
                    runningbus.setSafetyAlarm(1);
                } else {
                    runningbus.setSafetyAlarm(0);
                }
                gisRunningbusMapper.updateActiveSafety(runningbus.getSafetyAlarm(),runningbus.getId());
            }
        }
    }

    /**
     * 主动安全附件补充
     * @param dto
     */
    public void gisActiceSafetyFile(ActiveSafetyFileDto dto) {
        List<FileInfoDto> fileInfos = dto.getFileList();
        if (Objects.nonNull(fileInfos)) {
            GisActiveSafetyAlarm alarm = null;
            //重试次数
            int retries = 0;
            while (retries < 3) {
                LambdaQueryWrapper<GisActiveSafetyAlarm> queryWrapper = new LambdaQueryWrapper<>();
                queryWrapper.eq(GisActiveSafetyAlarm::getWarnNo, dto.getWarnNo());
                alarm = safetyAlarmMapper.selectOne(queryWrapper);

                if (Objects.nonNull(alarm)) {
                    // 找到alarm后退出循环
                    break;
                } else {
                    // 如果alarm为null，等待一段时间后重新尝试查询
                    try {
                        // 等待60秒
                        Thread.sleep(60*1000);
                    } catch (InterruptedException e) {
                        Thread.currentThread().interrupt();
                        // 在这里处理中断异常（如果需要）
                    }
                    retries++;
                }
            }
            if (Objects.nonNull(alarm)) {
                //文件类型 -1-图片 2-视频 3-语音
                for (FileInfoDto file : fileInfos) {
                    if (file.getFileType() == 0) {
                        alarm.setImgUrl(file.getFileUrl());
                    } else if (file.getFileType() == 2) {
                        alarm.setVideoUrl(file.getFileUrl());
                    } else if (file.getFileType() == 1) {
                        alarm.setVoiceUrl(file.getFileUrl());
                    }
                }
                safetyAlarmMapper.updateById(alarm);
            }
        }
    }

}
