package com.gci.cdenv.data.consume.app.modules.gps.service;

import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.gci.cdenv.data.consume.app.modules.common.ConstantKey;
import com.gci.cdenv.data.consume.app.modules.common.RedisKey;
import com.gci.cdenv.data.consume.app.modules.common.enums.WorkType;
import com.gci.cdenv.data.consume.app.modules.common.util.CdUrlUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.PlateNumberUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.PrimaryKeyUtil;
import com.gci.cdenv.data.consume.app.modules.gps.constant.GearPositionType;
import com.gci.cdenv.data.consume.app.modules.gps.constant.ProjectHandleType;
import com.gci.cdenv.data.consume.app.modules.gps.dto.*;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisGps;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisRunningbus;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.CommonMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisGpsMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author lyz
 * @date 2023/8/30 16:47
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class GpsConsumerService {

    @Autowired
    private GisFenceService gisFenceService;
    @Autowired
    private GisSpeedService gisSpeedService;

    @Autowired
    private CommonMapper commonMapper;

    @Autowired
    private GpsUpdateRunningBusService gpsUpdateRunningBusService;
    @Autowired
    private GisGpsMapper gisGpsMapper;

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Autowired
    private StringRedisTemplate redisTemplate3;
    @Autowired
    private CdUrlUtil cdUrlUtil;


    public void processLocationData(List<GpsDto> gpsDtos) {
        List<GisGps> gisGpsList = gpsDtos.parallelStream()
                .filter(gpsDto -> isValidGpsData(gpsDto))
                .map(gpsDto -> processValidGpsData(gpsDto))
                .collect(Collectors.toList());
        if (!gisGpsList.isEmpty()) {
            //插入业务数据库
            this.asyncInsert(gisGpsList);
            // 同步至外部redis
            this.asyncOuterRedisDTO(gisGpsList);
        }
    }

    private boolean isValidGpsData(GpsDto gpsDto) {
        // TODO: 2023/10/28 前面已經做判断，则不再做判断
//        if (gpsDto.isEic() && Objects.isNull(gpsDto.getVehicleDTO())) {
//            log.info("无效数据，整车数据没有，过滤来源为{}的eic记录", gpsDto.is808() ? "808" : "eic");
//            return false;
//        }
        VehicleInfoDTO vehicleInfoDTO = getVehicleInfo(gpsDto);

        if (Objects.isNull(vehicleInfoDTO)) {
//            log.info("终端设备定位设备标识-{}，查找车辆信息不存在，过滤来源为{}的gps记录", gpsDto.getData().getVtkey(), gpsDto.is808() ? "808" : "eic");
            return false;
        }
        if(!vehicleInfoDTO.isValidByTime(gpsDto.getData().getGpstime())){
//            log.info("gps记录时间异常，认定为无效。-{}",JSONObject.toJSONString(gpsDto));
            return false;
        }
        // 三电来源的数据始终有效
        if (gpsDto.isEic()) {
            return true;
        } else {
            //如果为天投项目的车辆，也返回true。因为需要特殊处理808的作业状态
            // TODO: 2023/10/23 后续所有车都走32960接入，则可以删除此段代码
            if (PlateNumberUtil.ttszWhiteList(vehicleInfoDTO.getProjectId(), vehicleInfoDTO.getProjectName())) {
                return true;
            }
            //如果为808来源的，则需判断该车是否支持三电，如果不支持三电  或者超过5分钟没有数据了。则参与计算和更新
            return vehicleInfoDTO.isThreeElectricWithRecentGPSUpdate(ConstantKey.DEFAULT_MINUTE);
        }
    }

    private GisGps processValidGpsData(GpsDto gpsDto) {
        VehicleInfoDTO vehicleInfoDTO = getVehicleInfo(gpsDto);
        //只有gps数据有效，且在最新gps时间之后。才做gps相关信息处理
        if (gpsDto.getData().isValid() && vehicleInfoDTO.needUpdate(gpsDto.getData().getGpstime())) {
            boolean outFence = gisFenceService.gisFenceAndBackOutFence(vehicleInfoDTO, gpsDto);
            boolean isSpeed = gisSpeedService.gisSpeedAndBacIsSpeed(vehicleInfoDTO, gpsDto);
            gpsUpdateRunningBusService.gpsUpdateRunningBus(vehicleInfoDTO, gpsDto, outFence, isSpeed);
        }
        return buildGisGps(vehicleInfoDTO, gpsDto);
    }

    private VehicleInfoDTO getVehicleInfo(GpsDto gpsDto) {
        if (gpsDto.isEic()) {
            return commonMapper.getVehicleInfoByVin(gpsDto.getData().getVtkey());
        } else {
            return commonMapper.getVehicleInfoByVtKey(gpsDto.getData().getVtkey());
        }
    }

    @Async
    public   void asyncOuterRedisDTO(List<GisGps> gisGpsList){
        //天投项目  崇环城服项目 的分发
        Map<String, List<GisGps>> collect = gisGpsList.stream()
                .collect(Collectors.groupingBy(g ->
                        PlateNumberUtil.getHandleType(g.getProjectId(), g.getProjectName())));
        collect.forEach((handleType,gisGps)->{
            if (!gisGps.isEmpty()) {
                extracted(gisGps,handleType);
            }
        });
    }

    private void extracted(List<GisGps> list, String type) {
        List<GpsOuterRedisDTO> outerRedisDTOs  = list.stream().map(gps->{
            GpsOuterRedisDTO dto = new GpsOuterRedisDTO();
            dto.setGuid("vehic"+gps.getPlateNumber());
            dto.setGpsTime(gps.getGpsTime().getTime());
            dto.setGpsValid(gps.getIsValid()==1);
            dto.setGpsCount(1);
            dto.setGpsLatitude(gps.getLat());
            dto.setGpsLongitude(gps.getLon());
            dto.setGpsSpeed(gps.getSpeed());
            dto.setGpsDirection(gps.getDir());
            dto.setGpsMileage(gps.getTotalMileage());
            dto.setGpsAltitude(gps.getAltitude());
            dto.setIgnitionStatus(gps.getAcc()==1);
//            dto.setFireStatus(false);
//            dto.setGpsStatus(false);
//            dto.setSpeed(gps.getSpeed());
            dto.setSwitching0(gps.getIsWork()==1);
//            dto.setIsNewVal(false);
//            dto.setMeasureType("");
//            dto.setRemainVal(BigDecimal.ZERO);
//            dto.setRemainUnit("");
//            dto.setUsedVal(BigDecimal.ZERO);
//            dto.setUsedUnit("");
            return dto;
        }).sorted(Comparator.comparing(GpsOuterRedisDTO::getGpsTime))
                .collect(Collectors.toList());
        switch (type) {
            case "1":
                List<String> dtoStrList = outerRedisDTOs.stream().map(JSONObject::toJSONString).collect(Collectors.toList());
                redisTemplate3.opsForList().rightPushAll(RedisKey.OUTER_GPS_QEUENAME,dtoStrList);
                break;
            case "2":
                String jsonString = JSONObject.toJSONString(outerRedisDTOs);
                try{
                    cdUrlUtil.pushGpsOil(jsonString);
                }catch (Exception e){
                    e.printStackTrace();
                    log.error("推送平台接口失败，数据大小：{}条，错误信息：{}",outerRedisDTOs.size(),e.getMessage());
                }
                break;
            default:
        }
    }

    @Async
    public void asyncInsert(List<GisGps> gisGpsList) {
        gisGpsMapper.batchInsertGisGps(gisGpsList);
    }

    /**
     * 组装gps入库数据
     *
     * @param vehicleInfoDTO
     * @param gpsDto
     * @return
     */
    public GisGps buildGisGps(VehicleInfoDTO vehicleInfoDTO, GpsDto gpsDto) {
        GisGps gisGps = new GisGps();
        GpsData data = gpsDto.getData();
        gisGps.setId(PrimaryKeyUtil.gengerateId());
        gisGps.setGpsTime(data.getGpstime());
        gisGps.setPlateNumber(vehicleInfoDTO.getPlateNumber());
        gisGps.setProjectId(vehicleInfoDTO.getProjectId());
        gisGps.setProjectName(vehicleInfoDTO.getProjectName());
        gisGps.setGpsNo(vehicleInfoDTO.getLocationTerminalId());
        gisGps.setLat(data.getLat());
        gisGps.setLon(data.getLon());
        gisGps.setSpeed(data.getSpeed());
        gisGps.setDir(String.valueOf(data.getDir()));
        gisGps.setAltitude(data.getAltitude());
        gisGps.setAcc(data.getAcc());
        gisGps.setStatus(data.getAcc() == 1 ? 1 : 2);
        gisGps.setRecordSource(gpsDto.getSource());
        gisGps.setCreateTime(new Date());
        gisGps.setIsValid(data.getIseff());
        gisGps.setIsWork(WorkType.INVALID.getCode());
        //补充三电数据
        if (gpsDto.isEic()) {
            VehicleDTO vehicleDTO = gpsDto.getVehicleDTO();
            gisGps.setTerminalConn(gpsDto.getTerminalConn());
            gisGps.setChargeStatus(vehicleDTO.getChargeStatus());
            //如果是三电记录，则更新标识为vincode.
            gisGps.setGpsNo(vehicleInfoDTO.getVinCode());
            gisGps.setOperatingMode(vehicleDTO.getOperatingMode());
            gisGps.setTotalMileage(vehicleDTO.getTotalMileage());
            gisGps.setTotalVoltage(vehicleDTO.getTotalVoltage());
            gisGps.setTotalCurrent(vehicleDTO.getTotalCurrent());
            gisGps.setStateOfCharge(vehicleDTO.getStateOfCharge().intValue());
            gisGps.setDcStatus(vehicleDTO.getDcDcStatus());
            gisGps.setGearPosition(GearPositionType.getNameByCode(vehicleDTO.getGearPosition()));
            // TODO: 2023/10/23 如果是三电数据过来的，则暂时从实时车辆信息里取作业状态,如果取不到则
            Object obj = redisTemplate.opsForHash().get(RedisKey.VEHICLE_REAL_INFO, vehicleInfoDTO.getPlateNumber());
            if (obj != null) {
                GisRunningbus bus = JSONObject.parseObject(obj.toString(), GisRunningbus.class);
                gisGps.setIsWork(bus.getIsWork());
//                long diff = gisGps.getGpsTime().getTime() - bus.getGpsTime().getTime();
//                //如果eic的采集时间跟实时车辆信息的时间小于3分分钟。则取实时车辆信息。
//                if (diff < 3 * 60 * 1000) {
//                    gisGps.setIsWork(bus.getIsWork());
//                } else {
//                    gisGps.setIsWork(0);
//                }
            }
        } else {
            gisGps.setTerminalConn(1);
            gisGps.setLowBeam(data.getLowBeam());
            gisGps.setHighBeam(data.getHighBeam());
            gisGps.setReverseGear(data.getReverseGear());
            gisGps.setIsWork(data.getWorkStatus(vehicleInfoDTO.isFuLongMa()));
        }
        return gisGps;
    }
}
