package com.gci.cdenv.data.consume.app.modules.gps.service;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.gci.cdenv.data.consume.app.modules.common.RedisKey;
import com.gci.cdenv.data.consume.app.modules.common.enums.WorkType;
import com.gci.cdenv.data.consume.app.modules.common.util.CoordinateConverterUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.LatLonPoint;
import com.gci.cdenv.data.consume.app.modules.gps.constant.AlarmType;
import com.gci.cdenv.data.consume.app.modules.gps.constant.GearPositionType;
import com.gci.cdenv.data.consume.app.modules.gps.dto.GpsData;
import com.gci.cdenv.data.consume.app.modules.gps.dto.GpsDto;
import com.gci.cdenv.data.consume.app.modules.gps.dto.VehicleDTO;
import com.gci.cdenv.data.consume.app.modules.gps.dto.VehicleInfoDTO;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisRunningbus;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.CommonMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisRunningbusMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * gps信息更新rubbingBus
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class GpsUpdateRunningBusService {

    @Autowired
    private GisRunningbusMapper gisRunningbusMapper;

    @Autowired
    private StringRedisTemplate redisTemplate;

    private final ConcurrentHashMap<String, Lock> vehicleLocks = new ConcurrentHashMap<>();

    @Async
    public void gpsUpdateRunningBus(VehicleInfoDTO vehicleInfoDTO, GpsDto gpsDto, boolean outFence, boolean isSpeed) {
        //获取车牌号
        String numberPlate = vehicleInfoDTO.getPlateNumber();
        GisRunningbus runningbus = gisRunningbusMapper.selectByPlateNumber(numberPlate);
        //更新车辆信息
        Lock lock = vehicleLocks.computeIfAbsent(numberPlate, key -> new ReentrantLock());
        lock.lock();
        try {
            if (Objects.nonNull(runningbus)) {
                buildRunningBus(vehicleInfoDTO, runningbus, gpsDto, outFence, isSpeed);
                gisRunningbusMapper.updateById(runningbus);
            } else {
                runningbus = new GisRunningbus();
                runningbus.setPlateNumber(numberPlate);
                //新增时需默认告警正常,作业状态无效，
                runningbus.setFaultAlarm(0);
                runningbus.setSafetyAlarm(0);
                runningbus.setIsWork(WorkType.INVALID.getCode());
                buildRunningBus(vehicleInfoDTO, runningbus, gpsDto, outFence, isSpeed);
                gisRunningbusMapper.insert(runningbus);
            }
            redisTemplate.opsForHash().put(RedisKey.VEHICLE_REAL_INFO, vehicleInfoDTO.getPlateNumber(), JSONObject.toJSONString(runningbus));
        } catch (Exception e) {
            e.printStackTrace();
            lock.unlock();
        } finally {
            lock.unlock();
        }
    }

    private Integer checkFaultAlarm(String plateNumber) {
        int integer = 0;
        AlarmType[] values = AlarmType.values();
        List<String> collect = new ArrayList<String>();
        for (AlarmType value : values) {
            collect.add(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp());
        }
        List<String> objects = redisTemplate.opsForValue().multiGet(collect);
        if (ObjectUtil.isNotEmpty(objects)) {
            for (String object : objects) {
                JSONObject jsonObject = JSONObject.parseObject(StrUtil.removeAll(StrUtil.strip(object, "\""),"\\"));
                if (ObjectUtil.isNotEmpty(jsonObject)) {
                    Boolean status = jsonObject.getBoolean("status");
                    if (status) {
                        integer = 1;
                        return integer;
                    }
                }
            }
        }
        return integer;
    }


    private void buildRunningBus(VehicleInfoDTO vehicleInfoDTO, GisRunningbus runningbus, GpsDto gpsDto, boolean outFence, boolean isSpeed) {
        GpsData gpsData = gpsDto.getData();
        runningbus.setStatus(gpsData.getAcc() == 1 ? 1 : 2);
        //如果该gps数据是通过三电组装而来,则可以获取三电相关的整车数据。
        runningbus.setGpsNo(vehicleInfoDTO.getLocationTerminalId());
        runningbus.setAdasNo(vehicleInfoDTO.getActiveSafetyDeviceId());
        runningbus.setVinCode(vehicleInfoDTO.getVinCode());
        runningbus.setProjectId(vehicleInfoDTO.getProjectId());
        if (gpsDto.isEic()) {
            VehicleDTO vehicleDTO = gpsDto.getVehicleDTO();
            if (Objects.nonNull(vehicleDTO)) {
                runningbus.setVehicleJson(JSONObject.toJSONString(vehicleDTO));
                runningbus.setStatus(vehicleDTO.getStatus());
                runningbus.setChargeStatus(vehicleDTO.getChargeStatus());
                runningbus.setOperatingMode(vehicleDTO.getOperatingMode());
                runningbus.setTotalVoltage(vehicleDTO.getTotalVoltage());
                runningbus.setTotalCurrent(vehicleDTO.getTotalCurrent());
                runningbus.setTotalMileage(vehicleDTO.getTotalMileage());
                runningbus.setDcStatus(vehicleDTO.getDcDcStatus());
                runningbus.setStateOfCharge(vehicleDTO.getStateOfCharge().intValue());
                runningbus.setGearPosition(GearPositionType.getNameByCode(vehicleDTO.getGearPosition()));
                runningbus.setSupportEic(1);
                runningbus.setChargeStatus(vehicleDTO.getChargeStatus());
            }
            runningbus.setEngineJson(Objects.isNull(gpsDto.getEngine()) ? null : JSONObject.toJSONString(gpsDto.getEngine()));
            runningbus.setAlarmDataJson(Objects.isNull(gpsDto.getAlarmData()) ? null : JSONObject.toJSONString(gpsDto.getAlarmData()));
            runningbus.setElectricMotorJson(Objects.isNull(gpsDto.getElectricMotor()) ? null : JSONObject.toJSONString(gpsDto.getElectricMotor()));
            runningbus.setExtremeDataJson(Objects.isNull(gpsDto.getExtremeData()) ? null : JSONObject.toJSONString(gpsDto.getExtremeData()));
            runningbus.setLocationJson(Objects.isNull(gpsDto.getLocation()) ? null : JSONObject.toJSONString(gpsDto.getLocation()));
            runningbus.setFaultAlarm(checkFaultAlarm(runningbus.getPlateNumber()));

            // TODO: 2023/10/23 32960过来的gps数据的暂不处理作业状态。
            // TODO: 2024/1/8 32960需要处理作业状态，因为会出现808一直没数据上来，32960需要把作业状态更正。
            if(runningbus.needUpdateWorkStatus(gpsData.getGpstime())){
                runningbus.setIsWork(WorkType.NOT_WORK.getCode());
            }
//            runningbus.setIsWork(gpsData.getWorkStatus());
        } else {
            runningbus.setGpsNo(gpsData.getVtkey());
            // TODO: 2023/10/23 更新作业状态，暂时只有808过来的gps才会更新
            runningbus.setIsWork(gpsData.getWorkStatus(vehicleInfoDTO.isFuLongMa()));
            runningbus.setWorkRecordTime(gpsData.getGpstime());
        }
        runningbus.setGpsTime(gpsData.getGpstime());
        //只有当有坐标信息，才更runningbus里的经纬度信息。
        if(ObjectUtil.isAllNotEmpty(gpsData.getLat(),gpsData.getLon())){
            runningbus.setLat(gpsData.getLat());
            runningbus.setLon(gpsData.getLon());
            //转换高德信息
            LatLonPoint point = CoordinateConverterUtil.toWGSPoint(gpsData.getLat().doubleValue(), gpsData.getLon().doubleValue());
            runningbus.setGdLon(new BigDecimal(point.getLongitude()));
            runningbus.setGdLat(new BigDecimal(point.getLatitude()));
        }
        runningbus.setSpeed(gpsData.getSpeed().setScale(2, BigDecimal.ROUND_HALF_UP));
        runningbus.setDir(String.valueOf(gpsData.getDir()));
        //在线状态为1
        runningbus.setIsOnline(1);
        runningbus.setAltitude(gpsData.getAltitude().intValue());
        runningbus.setAcc(gpsData.getAcc());
        runningbus.setSpeedingAlarm(isSpeed ? 1 : 0);
        runningbus.setFenceAlarm(outFence ? 1 : 0);
        runningbus.setRecordSource(gpsDto.getSource());
    }
}
