package com.gci.cdenv.data.consume.app.modules.gps.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.bean.DynaBean;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.gci.cdenv.data.consume.app.modules.common.RedisKey;
import com.gci.cdenv.data.consume.app.modules.common.util.CoordinateConverterUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.LatLonPoint;
import com.gci.cdenv.data.consume.app.modules.gps.constant.AlarmType;
import com.gci.cdenv.data.consume.app.modules.gps.constant.ChargeStatus;
import com.gci.cdenv.data.consume.app.modules.gps.dto.*;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisChargeLog;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisRunningbus;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisVehicleFaultAlarm;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.CommonMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisChargeLogMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisRunningbusMapper;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisVehicleFaultAlarmMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

/**
 * @author 24642
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class VehicleThreeElectricService {
    @Autowired
    private GisVehicleFaultAlarmMapper gisVehicleFaultAlarmMapper;
    @Autowired
    private GisRunningbusMapper gisRunningbusMapper;
    @Autowired
    private GisChargeLogMapper gisChargeLogMapper;
    @Autowired
    private CommonMapper commonMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;

    private final ConcurrentHashMap<String, Lock> vehicleLocks = new ConcurrentHashMap<>();

    private final ConcurrentHashMap<String, Lock> vehicleEicLocks = new ConcurrentHashMap<>();

    public void handle(List<VehicleThreeElectric> dtos) {
        dtos.parallelStream().forEach(vehicleThreeElectric -> {
            Date collectTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
            VehicleInfoDTO vehicleInfoByVin = commonMapper.getVehicleInfoByVin(vehicleThreeElectric.getVin());
            if (BeanUtil.isNotEmpty(vehicleInfoByVin) && vehicleInfoByVin.isValidByTime(collectTime)) {
                chargeLog(vehicleThreeElectric, vehicleInfoByVin);
                faultAlarm(vehicleThreeElectric, vehicleInfoByVin);
                //        eic(vehicleThreeElectric,vehicleInfoByVin);
            }
        });
    }

    public void eic(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin) {
        Lock lock = vehicleLocks.computeIfAbsent(vehicleInfoByVin.getPlateNumber(), key -> new ReentrantLock());
        lock.lock();
        try {
            String time = vehicleThreeElectric.getTime();
            DateTime cerTime = DateUtil.parse(time);
            String s = redisTemplate.opsForValue().get(RedisKey.VEHICLE_RUNNINGBUS_TIME + vehicleInfoByVin.getPlateNumber());
            if (StrUtil.isNotBlank(s)) {
                DateTime reTime = DateUtil.parse(s);
                //cerTime < relTime 才是正常情况,其他的不处理
                if (DateUtil.between(reTime, cerTime, DateUnit.MS, false) > 0) {
                    redisTemplate.opsForValue().set(RedisKey.VEHICLE_RUNNINGBUS_TIME + vehicleInfoByVin.getPlateNumber(), time);
                } else {
//                    log.warn("三电数据时间小于上次时间，原始数据:{}",JSONObject.toJSONString(vehicleThreeElectric));
                    return;
                }
            } else {
                redisTemplate.opsForValue().set(RedisKey.VEHICLE_RUNNINGBUS_TIME + vehicleInfoByVin.getPlateNumber(), time);
            }
            GisRunningbus gisRunningbus = gisRunningbusMapper.selectByPlateNumber(vehicleInfoByVin.getPlateNumber());
            //补充runningbus数据
            if (Objects.nonNull(gisRunningbus)) {
                buildGisRunningBus(gisRunningbus, vehicleThreeElectric, vehicleInfoByVin);
                gisRunningbusMapper.updateById(gisRunningbus);
            } else {
                gisRunningbus = new GisRunningbus();
                buildGisRunningBus(gisRunningbus, vehicleThreeElectric, vehicleInfoByVin);
                gisRunningbus.setGpsTime(cerTime);
                gisRunningbusMapper.insert(gisRunningbus);
            }
            redisTemplate.opsForHash().put(RedisKey.VEHICLE_REAL_INFO, vehicleInfoByVin.getPlateNumber(), JSONObject.toJSONString(gisRunningbus));
        } catch (Exception e) {
            lock.unlock();
            e.printStackTrace();
            log.error("三电数据消费出现未知错误,错误：{},当前数据已抛弃,原始数据：{}", e, JSONObject.toJSONString(vehicleThreeElectric));
        } finally {
            lock.unlock();
        }
    }

    private void buildGisRunningBus(GisRunningbus gisRunningbus, VehicleThreeElectric vehicleThreeElectric,
                                    VehicleInfoDTO vehicleInfoByVin) {
        VehicleDTO vehicle = vehicleThreeElectric.getVehicle();
        LocationDTO location = vehicleThreeElectric.getLocation();
        List<ElectricMotorDTO> electricMotor = vehicleThreeElectric.getElectricMotor();
        AlarmDataDTO alarmData = vehicleThreeElectric.getAlarmData();
        ExtremeDataDTO extremeData = vehicleThreeElectric.getExtremeData();
        EngineDTO engine = vehicleThreeElectric.getEngine();

        gisRunningbus.setPlateNumber(vehicleInfoByVin.getPlateNumber());
        gisRunningbus.setGpsNo(vehicleInfoByVin.getLocationTerminalId());
        gisRunningbus.setAdasNo(vehicleInfoByVin.getActiveSafetyDeviceId());

        gisRunningbus.setVinCode(vehicleInfoByVin.getVinCode());
        gisRunningbus.setProjectId(vehicleInfoByVin.getProjectId());

        if (Objects.nonNull(vehicle)) {
            gisRunningbus.setStatus(vehicle.getStatus());
            Integer chargeStatus = vehicle.getChargeStatus();
            gisRunningbus.setChargeStatus(chargeStatus);
            gisRunningbus.setOperatingMode(vehicle.getOperatingMode());
            gisRunningbus.setTotalMileage(vehicle.getTotalMileage());
            gisRunningbus.setTotalVoltage(vehicle.getTotalVoltage());
            gisRunningbus.setTotalCurrent(vehicle.getTotalCurrent());
            gisRunningbus.setStateOfCharge(vehicle.getStateOfCharge().intValue());
            gisRunningbus.setDcStatus(vehicle.getDcDcStatus());
            gisRunningbus.setVehicleJson(JSONObject.toJSONString(vehicle));
        }
        gisRunningbus.setEngineJson(JSONObject.toJSONString(engine));
        gisRunningbus.setAlarmDataJson(JSONObject.toJSONString(alarmData));
        gisRunningbus.setElectricMotorJson(JSONObject.toJSONString(electricMotor));
        gisRunningbus.setExtremeDataJson(JSONObject.toJSONString(extremeData));
        gisRunningbus.setLocationJson(JSONObject.toJSONString(location));
        gisRunningbus.setFaultAlarm(checkFaultAlarm(vehicleInfoByVin.getPlateNumber()));
        gisRunningbus.setRecordSource(2);
    }


    public void chargeLog(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin) {
        try {
            VehicleDTO vehicle = vehicleThreeElectric.getVehicle();
            LocationDTO location = vehicleThreeElectric.getLocation();
            if (!ObjectUtil.hasEmpty(vehicleThreeElectric, vehicleInfoByVin, vehicle)) {
                chargeLog(vehicleThreeElectric, vehicleInfoByVin, vehicle, location, vehicle.getChargeStatus());
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("充电数据消费出现未知错误:{},原始数据{}", e, JSONObject.toJSONString(vehicleThreeElectric));
        }
    }

    public void faultAlarm(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin) {
        try {
            LocationDTO location = vehicleThreeElectric.getLocation();
            AlarmDataDTO alarmData = vehicleThreeElectric.getAlarmData();
            if (!ObjectUtil.hasEmpty(vehicleThreeElectric, vehicleInfoByVin, alarmData)) {
                faultAlarm(vehicleThreeElectric, vehicleInfoByVin, location, alarmData);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("告警数据消费出现未知错误:{},原始数据{}", e, JSONObject.toJSONString(vehicleThreeElectric));
        }
    }

    private Integer checkFaultAlarm(String plateNumber) {
        int integer = 0;
        AlarmType[] values = AlarmType.values();
        List<String> collect = new ArrayList<String>();
        for (AlarmType value : values) {
            collect.add(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp());
        }
        List<String> objects = redisTemplate.opsForValue().multiGet(collect);
        if (ObjectUtil.isNotEmpty(objects)) {
            for (String object : objects) {
                JSONObject jsonObject = JSONObject.parseObject(StrUtil.removeAll(StrUtil.strip(object, "\""),"\\"));
                if (ObjectUtil.isNotEmpty(jsonObject)) {
                    Boolean status = jsonObject.getBoolean("status");
                    if (status) {
                        integer = 1;
                        return integer;
                    }
                }
            }
        }
        return integer;
    }

    private void faultAlarm(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, LocationDTO location, AlarmDataDTO alarmData) {
        String plateNumber = vehicleInfoByVin.getPlateNumber();

        //异常时间直接不处理
        if (!vehicleInfoByVin.isValidByTime(vehicleInfoByVin.getGpsTime())) {
            return;
        }

        DynaBean dynaBean = DynaBean.create(alarmData);
        AlarmType[] values = AlarmType.values();
        for (AlarmType value : values) {
            Object o = dynaBean.get(value.getProp());
            Integer integer = Convert.toInt(o);
            //先获取是否已有告警
            Object is = redisTemplate.opsForValue().get(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp());
            //正在告警
            if (integer == 1) {
                //可能存在第一次未初始化
                if (ObjectUtil.isEmpty(is)) {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("status", true);
                    jsonObject.put("time", vehicleThreeElectric.getTime());
                    jsonObject.put("endTime", vehicleThreeElectric.getTime());
                    //设置为已有告警
                    redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp(), jsonObject.toJSONString());
                    buildVehicleFaultAlarm(vehicleThreeElectric, vehicleInfoByVin, value, alarmData, location);
                } else {
                    JSONObject jsonObject = JSONObject.parseObject(StrUtil.removeAll(StrUtil.strip(Convert.toStr(is), "\""),"\\"));
                    Boolean aBoolean = jsonObject.getBoolean("status");
                    //没有告警
                    if (!aBoolean) {
                        String endTime = jsonObject.getString("endTime");
                        if (StrUtil.isNotBlank(endTime)) {
                            DateTime endDateTime = DateUtil.parseDateTime(endTime);
                            DateTime cerTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
                            //当前的时间和上一次的结束充电时间比较 小于0说明数据是乱的
                            if (DateUtil.between(endDateTime, cerTime, DateUnit.MS, false) < 0) {
                                return;
                            }
                        } else {
                            jsonObject.put("endTime", vehicleThreeElectric.getTime());
                        }
                        buildVehicleFaultAlarm(vehicleThreeElectric, vehicleInfoByVin, value, alarmData, location);
                        //设置为已有告警
                        jsonObject.put("status", true);
                        jsonObject.put("time", vehicleThreeElectric.getTime());
                        redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp(), jsonObject.toJSONString());
                    }
                }
            } else if (integer == 0) {
                //结束告警
                if (ObjectUtil.isEmpty(is)) {
//                    log.warn("没有告警但是没有上次告警状态，什么情况会出现？原始数据：{}", JSONObject.toJSONString(vehicleThreeElectric));
                } else {
                    JSONObject jsonObject = JSONObject.parseObject(StrUtil.removeAll(StrUtil.strip(Convert.toStr(is), "\""),"\\"));
                    Boolean aBoolean = jsonObject.getBoolean("status");
                    //已有告警
                    if (aBoolean) {
                        DateTime startTime = DateUtil.parseDateTime(jsonObject.getString("time"));
                        DateTime cerTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
                        //大于才是正常数据 开始时间<结束时间 其他的暂时不考虑
                        if (DateUtil.between(startTime, cerTime, DateUnit.MS, false) > 0) {
                            insertVehicleFaultAlarm(vehicleThreeElectric, vehicleInfoByVin, value);
                            jsonObject.put("status", false);
                            jsonObject.put("time", vehicleThreeElectric.getTime());
                            jsonObject.put("endTime", vehicleThreeElectric.getTime());
                            redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_FAULT_ALARM + plateNumber + ":" + value.getProp(), jsonObject.toJSONString());
                        }
                    }
                }
            }
        }
    }

    private void insertVehicleFaultAlarm(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, AlarmType value) {
        Object o = redisTemplate.opsForValue().get(RedisKey.VEHICLE_FAULT_ALARM + vehicleInfoByVin.getPlateNumber() + ":" + value.getProp());
        if (ObjectUtil.isEmpty(o)) {
//            log.info("结束告警但是redis没有告警记录，当上传无告警数据时为正常情况？当前数据：{}", JSONObject.toJSONString(vehicleThreeElectric));
            return;
        }
        DateTime dateTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
        GisVehicleFaultAlarm alarm = JSONObject.parseObject(Convert.toStr(o), GisVehicleFaultAlarm.class);
        alarm.setAlarmEndTime(dateTime);
        alarm.setIsClosed(1);
        alarm.setCloseTime(dateTime);
        gisVehicleFaultAlarmMapper.updateById(alarm);
    }

    private void buildVehicleFaultAlarm(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, AlarmType value, AlarmDataDTO alarmData, LocationDTO location) {
        GisVehicleFaultAlarm alarm = new GisVehicleFaultAlarm();
        DateTime cerTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
        alarm.setPlateNumber(vehicleInfoByVin.getPlateNumber());
        alarm.setVinCode(vehicleThreeElectric.getVin());
        alarm.setFaultName(value.getDescription());
        alarm.setProjectId(vehicleInfoByVin.getProjectId());
        alarm.setProjectName(vehicleInfoByVin.getProjectName());
        alarm.setFaultType(value.getTypeValue());
        alarm.setAlarmStartTime(cerTime);
        alarm.setIsClosed(0);
        alarm.setFaultLevel(alarmData.getMaxAlarmLevel());
        alarm.setCollectionTime(cerTime);
        alarm.setCreateTime(cerTime);
        BigDecimal latBigDecimal = new BigDecimal(0.0);
        BigDecimal lonBigDecimal = new BigDecimal(0.0);
        if (ObjectUtil.isNotEmpty(location)) {
            latBigDecimal = location.getLat();
            lonBigDecimal = location.getLon();
        } else {
            Object o = redisTemplate.opsForHash().get(RedisKey.VEHICLE_REAL_INFO, vehicleInfoByVin.getPlateNumber());
            if (ObjectUtil.isNotEmpty(o)) {
                GisRunningbus gisRunningbus = JSONObject.parseObject(Convert.toStr(o), GisRunningbus.class);
                latBigDecimal = NumberUtil.toBigDecimal(gisRunningbus.getLat());
                lonBigDecimal = NumberUtil.toBigDecimal(gisRunningbus.getLon());
            } else {
                log.error("三电告警时位置信息并且redis没有缓存runningbus数据,抛弃数据。默认经纬度0.0");
            }
        }
        alarm.setLat(latBigDecimal);
        alarm.setLon(lonBigDecimal);
        LatLonPoint latLonPoint = CoordinateConverterUtil.toWGSPoint(latBigDecimal.doubleValue(), lonBigDecimal.doubleValue());
        alarm.setGdLat(NumberUtil.toBigDecimal(latLonPoint.getLatitude()));
        alarm.setGdLon(NumberUtil.toBigDecimal(latLonPoint.getLongitude()));
        gisVehicleFaultAlarmMapper.insert(alarm);
        redisTemplate.opsForValue().set(RedisKey.VEHICLE_FAULT_ALARM + vehicleInfoByVin.getPlateNumber() + ":" + value.getProp(), JSONObject.toJSONString(alarm));
    }

    private void chargeLog(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, VehicleDTO vehicle, LocationDTO location, Integer chargeStatus) {
        String plateNumber = vehicleInfoByVin.getPlateNumber();

        //异常时间直接不处理
        if (!vehicleInfoByVin.isValidByTime(vehicleInfoByVin.getGpsTime())) {
            return;
        }

        //正在充电
        if (chargeStatus == ChargeStatus.PARKING_CHARGING.getCode()) {
            //先获取是否已有数据
            Object o = redisTemplate.opsForValue().get(RedisKey.VEHICLE_IS_CHARGE + plateNumber);
            //可能存在第一次未初始化
            if (ObjectUtil.isEmpty(o)) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("status", true);
                jsonObject.put("time", vehicleThreeElectric.getTime());
                jsonObject.put("endTime", vehicleThreeElectric.getTime());
                //设置为正在充电
                redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_CHARGE + plateNumber, jsonObject.toJSONString());
                buildStartGisChargeLog(vehicleThreeElectric, vehicleInfoByVin, vehicle, location);
            } else {
                JSONObject jsonObject = JSONObject.parseObject(Convert.toStr(o));
                Boolean aBoolean = jsonObject.getBoolean("status");
                //没有在充电
                if (!aBoolean) {
                    String endTime = jsonObject.getString("endTime");
                    if (StrUtil.isNotBlank(endTime)) {
                        DateTime endDateTime = DateUtil.parseDateTime(endTime);
                        DateTime cerTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
                        //当前的时间和上一次的结束充电时间比较 小于0说明数据是乱的
                        if (DateUtil.between(endDateTime, cerTime, DateUnit.MS, false) < 0) {
                            return;
                        }
                    } else {
                        jsonObject.put("endTime", vehicleThreeElectric.getTime());
                    }
                    //设置为正在充电
                    buildStartGisChargeLog(vehicleThreeElectric, vehicleInfoByVin, vehicle, location);
                    jsonObject.put("status", true);
                    jsonObject.put("time", vehicleThreeElectric.getTime());
                    redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_CHARGE + plateNumber, jsonObject.toJSONString());
                }//在充电就不处理了 属于连续状态
            }
        } else if (chargeStatus == ChargeStatus.NOT_CHARGING.getCode() || chargeStatus == ChargeStatus.CHARGING_COMPLETED.getCode()) {
            //充电完成或者 未充电 属于结束充电了
            //先获取是否已有数据
            Object o = redisTemplate.opsForValue().get(RedisKey.VEHICLE_IS_CHARGE + plateNumber);
            if (ObjectUtil.isEmpty(o)) {
//                log.warn("结束充电但是没有上一次充电状态,什么情况会出现？当前数据:{}", JSONObject.toJSONString(vehicleThreeElectric));
            } else {
                JSONObject jsonObject = JSONObject.parseObject(Convert.toStr(o));
                Boolean aBoolean = jsonObject.getBoolean("status");
                if (aBoolean) {
                    DateTime startTime = DateUtil.parseDateTime(jsonObject.getString("time"));
                    DateTime cerTime = DateUtil.parseDateTime(vehicleThreeElectric.getTime());
                    //大于才是正常数据 开始时间<结束时间 其他的暂时不考虑
                    if (DateUtil.between(startTime, cerTime, DateUnit.MS, false) > 0) {
                        //已经有充电记录了 现在要结束充电
                        insertEndGisChargeLog(vehicleThreeElectric, vehicleInfoByVin, vehicle);
                        jsonObject.put("status", false);
                        jsonObject.put("time", "");
                        jsonObject.put("endTime", vehicleThreeElectric.getTime());
                        redisTemplate.opsForValue().set(RedisKey.VEHICLE_IS_CHARGE + plateNumber, jsonObject.toJSONString());
                    } else {
//                        log.warn("结束充电但是时间比开始时间早，不处理数据：{}", JSONObject.toJSONString(vehicleThreeElectric));
                    }
                }
            }
        }
    }

    /**
     * 构建并插入结束时候的记录
     */
    private void insertEndGisChargeLog(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, VehicleDTO vehicle) {
        Lock lock = vehicleEicLocks.computeIfAbsent(vehicleInfoByVin.getPlateNumber(), key -> new ReentrantLock());
        lock.lock();
        try {
            Object o = redisTemplate.opsForValue().get(RedisKey.VEHICLE_CHARGE_LOG + vehicleInfoByVin.getPlateNumber());
        if (ObjectUtil.isEmpty(o)) {
            log.info("结束充电但是没有Redis充电记录,当上传未充电数据时为正常情况？当前数据:{}", JSONObject.toJSONString(vehicle));
            return;
        }
        Date date = DateUtil.parse(vehicleThreeElectric.getTime());
        GisChargeLog gisChargeLog = JSONObject.parseObject(Convert.toStr(o), GisChargeLog.class);
        gisChargeLog.setEndTime(date);
        gisChargeLog.setEndSoc(vehicle.getStateOfCharge());
        BigDecimal sub = NumberUtil.sub(gisChargeLog.getEndSoc(), gisChargeLog.getStartSoc());
        gisChargeLog.setChargingEfficiency(sub);
        BigDecimal decimal = NumberUtil.div(sub, 100, 2);
        gisChargeLog.setChargingEnergy(NumberUtil.round(NumberUtil.mul(gisChargeLog.getTotalBatteryEnergy(), decimal), 2));
        gisChargeLog.setCreateTime(new Date());
        gisChargeLogMapper.insert(gisChargeLog);
        redisTemplate.delete(RedisKey.VEHICLE_CHARGE_LOG + vehicleInfoByVin.getPlateNumber());
        } catch (Exception e) {
            e.printStackTrace();
            lock.unlock();
        } finally {
            lock.unlock();
        }
    }

    /**
     * 构建开始时候的记录
     */
    private void buildStartGisChargeLog(VehicleThreeElectric vehicleThreeElectric, VehicleInfoDTO vehicleInfoByVin, VehicleDTO vehicle, LocationDTO location) {
        GisChargeLog gisChargeLog = new GisChargeLog();
        Date date = DateUtil.parse(vehicleThreeElectric.getTime());
        gisChargeLog.setNumberPlate(vehicleInfoByVin.getPlateNumber());
        gisChargeLog.setProjectId(vehicleInfoByVin.getProjectId());
        gisChargeLog.setProjectName(vehicleInfoByVin.getProjectName());
        gisChargeLog.setVinCode(vehicleInfoByVin.getVinCode());
        gisChargeLog.setStartTime(date);
        gisChargeLog.setStartSoc(vehicle.getStateOfCharge());

        gisChargeLog.setTotalBatteryEnergy(NumberUtil.toBigDecimal(vehicleInfoByVin.getBatteryCapacity()));
        BigDecimal latBigDecimal = new BigDecimal(0.0);
        BigDecimal lonBigDecimal = new BigDecimal(0.0);
        if (ObjectUtil.isNotEmpty(location)) {
            latBigDecimal = NumberUtil.toBigDecimal(location.getLat());
            lonBigDecimal = NumberUtil.toBigDecimal(location.getLon());
        } else {
            Object o = redisTemplate.opsForHash().get(RedisKey.VEHICLE_REAL_INFO, vehicleInfoByVin.getPlateNumber());
            if (ObjectUtil.isNotEmpty(o)) {
                GisRunningbus gisRunningbus = JSONObject.parseObject(Convert.toStr(o), GisRunningbus.class);
                latBigDecimal = NumberUtil.toBigDecimal(gisRunningbus.getLat());
                lonBigDecimal = NumberUtil.toBigDecimal(gisRunningbus.getLon());
            } else {
                log.error("三电无充电时位置信息并且redis没有缓存runningbus数据,抛弃数据。默认经纬度0.0");
            }
        }
        gisChargeLog.setLat(latBigDecimal);
        gisChargeLog.setLon(lonBigDecimal);

        LatLonPoint latLonPoint = CoordinateConverterUtil.toWGSPoint(latBigDecimal.doubleValue(), lonBigDecimal.doubleValue());
        gisChargeLog.setGdLat(NumberUtil.toBigDecimal(latLonPoint.getLatitude()));
        gisChargeLog.setGdLon(NumberUtil.toBigDecimal(latLonPoint.getLongitude()));
        gisChargeLog.setCollectionTime(date);
        redisTemplate.opsForValue().set(RedisKey.VEHICLE_CHARGE_LOG + vehicleInfoByVin.getPlateNumber(), JSONObject.toJSONString(gisChargeLog));
    }

    public List<GpsDto> buildGpsDto(List<VehicleThreeElectric> dtos) {
        List<GpsDto> gpsDtos = new ArrayList<>(dtos.size());
        dtos.stream().forEach(vehicleThreeElectric -> {
            LocationDTO location = vehicleThreeElectric.getLocation();
            VehicleDTO vehicle = vehicleThreeElectric.getVehicle();
            GpsDto gpsDto = new GpsDto();
            gpsDto.setTerminalConn(vehicleThreeElectric.getTerminalConn());
            gpsDto.setVehicleDTO(vehicle);
            gpsDto.setElectricMotor(vehicleThreeElectric.getElectricMotor());
            gpsDto.setAlarmData(vehicleThreeElectric.getAlarmData());
            gpsDto.setEngine(vehicleThreeElectric.getEngine());
            gpsDto.setExtremeData(vehicleThreeElectric.getExtremeData());
            gpsDto.setLocation(location);
            gpsDto.setSource(2);
            GpsData gpsData = new GpsData();
            gpsData.setVtkey(vehicleThreeElectric.getVin());
            gpsData.setAcc(vehicle.getStatus());
            gpsData.setSpeed(vehicle.getSpeed());
            // TODO: 2023/10/28 改用整车数据判断,因为前面已经做了整车数据判断，所以进来这里必定为1
            gpsData.setIseff(1);
            gpsData.setDir(0);
            gpsData.setAltitude(new BigDecimal(0));
            gpsData.setGpstime(DateUtil.parseDateTime(vehicleThreeElectric.getTime()));
            if (BeanUtil.isNotEmpty(location)) {
                gpsData.setLat(location.getLat());
                gpsData.setLon(location.getLon());
                // TODO: 2023/10/28 不以三电的坐标数据是否有效来判断整条记录是否有效，改用整车数据判断
//                    gpsData.setIseff(location.getIsValid()==0?1:0);
            }
            gpsDto.setTime(vehicleThreeElectric.getTime());
            gpsDto.setData(gpsData);
            gpsDtos.add(gpsDto);
        });
        return gpsDtos;
    }
}
