package com.gci.cdenv.data.consume.app.modules.task;

import com.alibaba.fastjson.JSONObject;
import com.gci.cdenv.data.consume.app.modules.common.RedisKey;
import com.gci.cdenv.data.consume.app.modules.gps.dto.ActiveSafetyFileDto;
import com.gci.cdenv.data.consume.app.modules.gps.service.GisActiveSafetyService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

/**
 * @author lyz
 * @date 2023/8/31 17:03
 */
@Configuration
@EnableScheduling
@Slf4j
@RequiredArgsConstructor
public class FileInfoTask {

    @Autowired
    private  StringRedisTemplate redisTemplate;

    @Autowired
    private  GisActiveSafetyService gisActiveSafetyService;;

    @Value("${dataApp.fileTask.enabled}")
    private boolean fileEnabled;

    /**
     * 消费主动安全事件的附件
     */
    @Scheduled(fixedRateString = "${dataApp.fileTask.interval}")
    public void consumeActSafetyFileData() {
        // 如果启用了消费
        if (fileEnabled) {
            String fileInfo = redisTemplate.opsForList().leftPop(RedisKey.GCI_SB_FILE);
            log.info("定时任务消费adas附件，数据-{}",fileInfo);
            if (fileInfo != null) {
                try{
                    ActiveSafetyFileDto dto = JSONObject.parseObject(fileInfo,ActiveSafetyFileDto.class);
                    gisActiveSafetyService.gisActiceSafetyFile(dto);
                }catch (Exception e){
                    e.printStackTrace();
                    log.error("消费主动安全事件的附件失败，{}",fileInfo);
                }
            }
        }
    }
}
