package com.gci.cdenv.data.consume.app.modules.task;

import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.gci.cdenv.data.consume.app.modules.common.RedisKey;
import com.gci.cdenv.data.consume.app.modules.common.util.CoordinateConverterUtil;
import com.gci.cdenv.data.consume.app.modules.common.util.LatLonPoint;
import com.gci.cdenv.data.consume.app.modules.gps.dto.*;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisRunningbus;
import com.gci.cdenv.data.consume.app.modules.gps.entity.GisSpeedingAlarm;
import com.gci.cdenv.data.consume.app.modules.gps.mapper.GisRunningbusMapper;
import com.gci.cdenv.data.consume.app.modules.gps.vo.SpeedRedisVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author lyz
 * @date 2023/9/9 14:33
 */
@Configuration
@EnableScheduling
@Slf4j
public class RunningBusOnlineTask {

    @Autowired
    private GisRunningbusMapper gisRunningbusMapper;

    @Value("${dataApp.runningBusTask.enabled}")
    private boolean enabled;

    @Value("${dataApp.updateAlarmTask.enabled}")
    private boolean updateAlarmEnabled;

    @Value("${dataApp.updateAlarmTask.timeOut}")
    private Integer timeOut;

    @Value("${param.standSpeed}")
    private Integer standSpeed;

    @Autowired
    private StringRedisTemplate redisTemplate;

    /**
     * 消费
     */
    @Scheduled(fixedRateString = "${dataApp.runningBusTask.interval}")
    public void consumeActSafetyFileData() {
        log.info("车辆在线状态更新任务开始---");
        // 如果启用了消费
        if (enabled) {
            gisRunningbusMapper.updateOnlineStatusToZero(new Date());
        }
    }


    /**
     * 5分钟执行一次
     */
//    @Scheduled(fixedRate = 5 * 60 * 1000)
    public void updateFenceAlarmRecord() {
        log.info("车辆围栏告警关闭检测开始---");
        try {
            Long milliSecond = LocalDateTime.now().toInstant(ZoneOffset.of("+8")).toEpochMilli();
            //返回所有数据,判断时间戳是否小于的当前时间戳，进行删除
            Set<String> typedTuples = redisTemplate.opsForZSet().rangeByScore(RedisKey.FENCE_VEHICLE_SCORE, 0, milliSecond);
            if (typedTuples.size() > 0) {
                Iterator<String> iterator = typedTuples.iterator();
                while (iterator.hasNext()){
                    String typedTuple = iterator.next();
                    List<String> idList = Arrays.asList(typedTuple.split("-"));
                    String plateNumber = idList.get(0);
                    String alarmId = idList.get(1);
                    String fenceId = idList.get(2);
                    Object obj = redisTemplate.opsForHash().get(RedisKey.VEHICLE_REAL_INFO,plateNumber);
                    if(obj != null){
                        GisRunningbus bus = JSONObject.parseObject(obj.toString(), GisRunningbus.class);
                        bus.setFenceAlarm(0);
                        redisTemplate.opsForHash().put(RedisKey.VEHICLE_REAL_INFO, plateNumber, JSONObject.toJSONString(bus));
                    }
                    redisTemplate.delete(RedisKey.INSIDE_FENCE + plateNumber + fenceId);
                    redisTemplate.delete(RedisKey.CONSECUTIVE_INSIDE_COUNT + plateNumber + fenceId);
                    redisTemplate.delete(RedisKey.CONSECUTIVE_OUT_COUNT + plateNumber + fenceId);
                    redisTemplate.opsForZSet().remove(RedisKey.FENCE_VEHICLE_SCORE,typedTuple);
                    gisRunningbusMapper.updateFenceAlarmRecord(new Date(),alarmId);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("根据围栏生效时间关闭定时任务执行失败,", e.getMessage());
        }

    }

    /**
     * 车辆掉线超过10分钟，处理车辆的告警信息
     */
    @Scheduled(fixedDelayString = "${dataApp.updateAlarmTask.interval}")
    public void updateAlarmRecord() {
        log.info("车辆围掉线检测开始---");
        if (!updateAlarmEnabled) {
            return;
        }
        Map<Object, Object> vehiciles = redisTemplate.opsForHash().entries(RedisKey.VEHICLE_REAL_INFO);
        vehiciles.entrySet().forEach(v -> {
            String plateNumber = v.getKey().toString();
            GisRunningbus bus = JSONObject.parseObject(v.getValue().toString(), GisRunningbus.class);
            if (bus.offOnlineTimeOut(timeOut)) {
                //超过时间自动关闭围栏告警
//                processFenceAlarm(bus, plateNumber);
                processSpeedingAlarm(bus, plateNumber);
            }
        });
    }

    private void processFenceAlarm(GisRunningbus bus, String plateNumber) {
        if (Objects.nonNull(bus.getFenceAlarm()) && bus.getFenceAlarm() == 1) {
            FenceAlarm fenceAlarm = createFenceAlarm(plateNumber);
            Alarm alarm = createAlarm(1, 2, fenceAlarm);
            updateBusFenceAlarm(bus, plateNumber, alarm);
            pushAlarmToQueue(alarm);
        }
    }

    private void processSpeedingAlarm(GisRunningbus bus, String plateNumber) {
        if (Objects.nonNull(bus.getSpeedingAlarm()) && bus.getSpeedingAlarm() == 1) {
            SpeedRedisVO speedRedis = retrieveSpeedRedis(plateNumber);
            Alarm alarm = createSpeedingAlarm(bus, speedRedis);
            updateBusSpeedingAlarm(bus, plateNumber, alarm);
            pushAlarmToQueue(alarm);
        }
    }

    private FenceAlarm createFenceAlarm(String plateNumber) {
        FenceAlarm fenceAlarm = new FenceAlarm();
//        fenceAlarm.setFenceType(fenceDTO.getFenceType());
//        fenceAlarm.setName(fenceDTO.getFenceName());
        fenceAlarm.setPlateNumber(plateNumber);
        fenceAlarm.setAlarmTime(new Date());
        return fenceAlarm;
    }

    private Alarm createAlarm(int type, int flag, Object data) {
        Alarm alarm = new Alarm();
        alarm.setType(type);
        alarm.setFlag(flag);
        alarm.setTime(new Date());
        alarm.setData(JSONObject.parseObject(JSONObject.toJSONString(data)));
        return alarm;
    }


    private void updateBusFenceAlarm(GisRunningbus bus, String plateNumber, Alarm alarm) {
        bus.setFenceAlarm(0);
        redisTemplate.opsForHash().put(RedisKey.VEHICLE_REAL_INFO, plateNumber, JSONObject.toJSONString(bus));
        //获取车辆对应的围栏信息
        List<FenceDTO> fenceDTOS = Optional.ofNullable(redisTemplate.opsForValue().get(RedisKey.VEHICLE_FENCE_KEY + plateNumber)).map(a -> {
            return JSONUtil.toList(StrUtil.removeAll(StrUtil.strip(a, "\""), "\\"), FenceDTO.class).stream().collect(Collectors.toList());
        }).orElse(new ArrayList<>());
        //处理围栏车辆对应的缓存数据。
        for (FenceDTO fence : fenceDTOS) {
            redisTemplate.delete(RedisKey.INSIDE_FENCE + plateNumber + fence.getFenceId());
            redisTemplate.delete(RedisKey.CONSECUTIVE_INSIDE_COUNT + plateNumber + fence.getFenceId());
            redisTemplate.delete(RedisKey.CONSECUTIVE_OUT_COUNT + plateNumber + fence.getFenceId());
        }
    }

    private void updateBusSpeedingAlarm(GisRunningbus bus, String plateNumber, Alarm alarm) {
        bus.setSpeedingAlarm(0);
        //清理redis数据
        redisTemplate.delete(RedisKey.GIS_CONSECUTIVE_SPEED_COUNT + plateNumber);
        redisTemplate.opsForHash().put(RedisKey.VEHICLE_REAL_INFO, plateNumber, JSONObject.toJSONString(bus));
    }

    private void pushAlarmToQueue(Alarm alarm) {
        redisTemplate.opsForList().leftPush(RedisKey.FENCE_SPEED_ALARM_BEAN, JSONUtil.toJsonPrettyStr(alarm));
    }

    private SpeedRedisVO retrieveSpeedRedis(String plateNumber) {
        String redisStr = redisTemplate.opsForValue().get(RedisKey.GIS_CONSECUTIVE_SPEED_COUNT + plateNumber);
        return StrUtil.isNotBlank(redisStr) ? JSONUtil.toBean(redisStr, SpeedRedisVO.class) : null;
    }

    private Alarm createSpeedingAlarm(GisRunningbus bus, SpeedRedisVO speedRedis) {
        Alarm alarm = new Alarm();
        alarm.setType(2);
        alarm.setFlag(2);
        alarm.setTime(new Date());
        GisSpeedingAlarm speedAlarm = new GisSpeedingAlarm();
        speedAlarm.setPlateNumber(bus.getPlateNumber());
        speedAlarm.setProjectId(bus.getProjectId());
        speedAlarm.setGpsNo(bus.getGpsNo());
        speedAlarm.setRuleSpeed(BigDecimal.valueOf(standSpeed));

        List<Double> speedList = speedRedis != null ? speedRedis.getSpeedList() : new ArrayList<>();
        speedList.add(bus.getSpeed().doubleValue());
        speedAlarm.setSpeed(BigDecimal.valueOf(Collections.max(speedList)));
        speedAlarm.setAverageSpeed(BigDecimal.valueOf(speedList.stream().mapToDouble(Double::doubleValue).average().orElse(0)));

        LatLonPoint point = CoordinateConverterUtil.toWGSPoint(bus.getLat().doubleValue(), bus.getLon().doubleValue());
        speedAlarm.setEndLon(new BigDecimal(point.getLongitude()));
        speedAlarm.setEndLat(new BigDecimal(point.getLatitude()));
        speedAlarm.setEndTime(new Date());

        alarm.setData(JSONObject.parseObject(JSONObject.toJSONString(speedAlarm)));
        return alarm;
    }
}
